use super::{
    req_props::{Props, ReqProps, Response},
    service::Service,
};
use std::sync::Arc;
use tokio::sync::Mutex;

pub const DEFAULT_DEPTH: usize = 3;

pub struct Enumerator<'a> {
    service: &'a Service,
    path: String,
    depth: usize,
    properties: Vec<Props>,
}

impl<'a> Enumerator<'a> {
    pub fn new(service: &'a Service) -> Self {
        Enumerator {
            service,
            path: String::new(),
            depth: DEFAULT_DEPTH,
            properties: Vec::new(),
        }
    }

    pub fn set_path(mut self, path: String) -> Self {
        self.path = path;
        self
    }

    pub fn set_depth(mut self, depth: usize) -> Self {
        self.depth = depth;
        self
    }

    pub fn get_properties(mut self, properties: Vec<Props>) -> Self {
        self.properties.extend(properties);
        self
    }

    pub async fn enumerate(&self) -> Result<(Vec<Response>, Vec<Response>), std::io::Error> {
        let (tx, mut rx) = tokio::sync::mpsc::unbounded_channel();
        let files = Arc::new(Mutex::new(Vec::new()));
        let folders = Arc::new(Mutex::new(Vec::new()));
        let service = Arc::from(self.service.clone());

        tx.send(self.path.clone());

        let mut handles = vec![];

        while let Some(path) = rx.recv().await {
            let tx_clone = tx.clone();
            let files_clone = Arc::clone(&files);
            let folders_clone = Arc::clone(&folders);
            let service_clone = Arc::clone(&service);
            let properties = self.properties.clone();

            let handle = tokio::task::spawn_blocking(move || async move {
                let res = ReqProps::new(&service_clone)
                    .set_path(path)
                    // .set_depth(self.depth)
                    .get_properties(properties)
                    .send()
                    .await
                    .unwrap();

                for obj in res.responses {
                    if obj.is_dir() {
                        todo!("Deal to have good href");
                        tx_clone.send(obj.href.clone());
                        folders_clone.lock().await.push(obj);
                    } else {
                        files_clone.lock().await.push(obj);
                    }
                }
            });
            handles.push(handle);
        }

        // Wait for all tasks to complete
        for handle in handles {
            handle.await;
        }

        Ok((
            Arc::try_unwrap(files).unwrap().into_inner(),
            Arc::try_unwrap(folders).unwrap().into_inner(),
        ))
    }
}
